#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {


void VertexNormalData::setNormalOfFace(const SFVec3f &normal, BaseSFFace *face)
{
  // iterating the normals to find whether another one is identical.
  unsigned short i = 0;
  for (MFVec3f::iterator it = _normalArray.begin(); it != _normalArray.end(); ++it)
  {
    // found an identical normal.
    if (*it == normal)
    {
      _normalMap[face] = i;
      return;
    }
    ++i;
  }  
  // did not found an identical normal.
  _normalMap[face] = _normalArray.size();
  _normalArray.push_back(normal);
}

const SFVec3f &VertexNormalData::getNormalOfFace(BaseSFFace *face) const
{
  std::map<BaseSFFace *, unsigned short>::const_iterator it = _normalMap.find(face);
  if (it != _normalMap.end())
    return _normalArray[(*it).second];
    
  return SFVec3f::null;   
}

void VertexColorData::setColorOfFace(const SFColorRGBA &color, BaseSFFace *face)
{
  unsigned short i = 0;
  for (MFColorRGBA::iterator it = _colorArray.begin(); it != _colorArray.end(); ++it)
  {
    if (*it == color)
    {
      _colorMap[face] = i;
      return;
    }
    ++i;
  }  
  _colorMap[face] = _colorArray.size();
  _colorArray.push_back(color);  
}

const SFColorRGBA &VertexColorData::getColorOfFace(BaseSFFace *face) const
{
  std::map<BaseSFFace *, unsigned short>::const_iterator it = _colorMap.find(face);
  if (it != _colorMap.end())
    return _colorArray[(*it).second];
    
  return SFColorRGBA::null;  
}

void VertexTexCoordData::setTexCoordOfFace(const SFPoint2f &texCoord, BaseSFFace *face)
{
  unsigned short i = 0;
  for (MFPoint2f::iterator it = _texCoordArray.begin(); it != _texCoordArray.end(); ++it)
  {
    if (*it == texCoord)
    {
      _texCoordMap[face] = i;
      return;
    }
    ++i;
  }  
  _texCoordMap[face] = _texCoordArray.size();
  _texCoordArray.push_back(texCoord); 
}

const SFPoint2f &VertexTexCoordData::getTexCoordOfFace(BaseSFFace *face) const
{
  std::map<BaseSFFace *, unsigned short>::const_iterator it = _texCoordMap.find(face);
  if (it != _texCoordMap.end())
    return _texCoordArray[(*it).second];
    
  return SFPoint2f::null;
}

}
}
