#include "X3D_SharedCoordComputerRenderingVisitor.h"
#include "X3D_Coordinate.h"
#include "X3D_IndexedFaceSet.h"

#include <iostream>
#include <list>

using namespace std;

namespace X3DTK {
namespace X3D {

SharedCoordComputerRenderingVisitor::SharedCoordComputerRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  define(Recorder<Coordinate>::getEnterFunction(&SharedCoordComputerRenderingVisitor::enterCoordinate));
}

void SharedCoordComputerRenderingVisitor::enterCoordinate(Coordinate *C) 
{
  SharedCoordComputerStateVariables *stateVariables = Singleton<SharedCoordComputerStateVariables>::getInstance();

  if (!stateVariables->getNode(C))
  {
    // Checking the number of IndexedFaceSet parents.
    MFNode parentList = C->getParentList();
    list<IndexedFaceSet *> ifslist;
    for (MFNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
    {
      if (dynamic_cast<IndexedFaceSet *>(*it) != 0) 
      {
        ifslist.push_back(static_cast<IndexedFaceSet *>(*it));
      } 
    }  

    if (ifslist.size() > 1)
    {
      map<IndexedFaceSet *, std::pair<bool, list<SFNode> > > &ifsmap = stateVariables->find(ifslist);
      
      // Removing the parents of C.
      for (map<IndexedFaceSet *, std::pair<bool, list<SFNode> > >::const_iterator itPMap = ifsmap.begin(); itPMap != ifsmap.end(); ++itPMap)
        (*itPMap).first->removeChild(C);
      
      // processing the IndexedFaceSet.
      bool cloned = false;
      unsigned int n = 0;
      map<IndexedFaceSet *, std::pair<bool, list<SFNode> > >::iterator itMap = ifsmap.begin();
      while (itMap != ifsmap.end())
      {
        if (!(*itMap).second.first)
        {
          SFNode NC;
          if (!cloned)
          {
            NC = C;
            cloned = true;
          }  
          else
          {
            NC = C->clone();  
            NC->setName("");
            ++n;
          }  
        
          (*itMap).first->setChild(NC);
          
          map<IndexedFaceSet *, std::pair<bool, list<SFNode> > >::iterator itF = itMap;
          ++itF;
          while (itF != ifsmap.end())
          {
            if (!(*itF).second.first)
            {
              // testing compatibility between (*it).first and (*itF).first
              list<SFNode> l = (*itMap).second.second;
              list<SFNode> lF = (*itF).second.second;
              
              if ((l.size() == 1) && (lF.size() == 1) && (l.front() == lF.front()))
              {
                //cout << "fusion " << (*it).first << " and " << (*itF).first << endl;
                (*itF).first->setChild(NC);
                (*itF).second.first = true;
              }
            }
            ++itF;
          }
        }  
        ++itMap;
      }
    }
    stateVariables->addNode(C);
  }
}

}
}
